module m_parametros

! ::: MÓDULO: m_parametros :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!      Este módulo recoge tanto los parámetros del modelo de HH para el axón gigante de calamar como las declaraciones de variables globales 
!   usadas en el resto de programas.
!
! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

public

integer, parameter :: double = selected_real_kind(14)
integer, parameter :: dp = double

! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
! ::::::::::::::::::::::::::::: PARÁMETROS NEURONALES PARA EL MODELO DE HH

! Axón gigante de calamar (condiciones fisiológicas; T = 6.7ºC, [K+]ext = 10 mM, [K+]int = 244 mM).

real(kind=dp), parameter               :: C_mem  = 1.0_dp         ! μF/cm^2
real(kind=dp), parameter               :: V_rest = -64.030856_dp  ! mV (con E_K = -77 mV)
real(kind=dp)                          :: I_ext  = 0.0_dp         ! μA/cm^2

real(kind=dp), parameter               :: E_Na   = 55.0_dp        ! mV
real(kind=dp), parameter               :: g_Na   = 120.0_dp       ! mS/cm^2

real(kind=dp)                          :: E_K    = -77.0_dp       ! mV
real(kind=dp), parameter               :: g_K    = 36.0_dp        ! mS/cm^2

real(kind=dp), parameter               :: E_L    = -54.4_dp       ! mV
real(kind=dp), parameter               :: g_L    = 0.3_dp         ! mS/cm^2

real(kind=dp), dimension(3), parameter :: V_half  = (/ -40.0_dp, -62.0_dp, -53.0_dp /)  ! (/m, h, n/)
real(kind=dp), dimension(3), parameter :: k_slope = (/ 9.0_dp  , -7.0_dp , 15.0_dp  /)
real(kind=dp), dimension(3), parameter :: V_max   = (/ -38.0_dp, -67.0_dp, -79.0_dp /)
real(kind=dp), dimension(3), parameter :: sigma   = (/ 30.0_dp , 20.0_dp , 50.0_dp  /)
real(kind=dp), dimension(3), parameter :: C_amp   = (/ 0.46_dp , 7.4_dp  , 4.7_dp   /)
real(kind=dp), dimension(3), parameter :: C_base  = (/ 0.04_dp , 1.2_dp  , 1.1_dp   /)

!                                                                                                                    :::::::::::::::::::::::::::::
! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
! ::::::::::::::::::::::::::::: VARIABLES PARA LOS CÁLCULOS NUMÉRICOS Y LA REPRESENTACIÓN DE GRÁFICAS

real(kind=dp), parameter                :: PRECISION_SP = 1.0e-7_dp
real(kind=dp), parameter                :: PRECISION_DP = 1.0e-14_dp

real(kind=dp), dimension(2)             :: pto_eq, marker_coordinates
logical                                 :: diagrama_bif_total

real(kind=dp), dimension(5), parameter  :: rango = (/-75.949371467424_dp, & ! Bif. SN de ciclos límite
                                                     -73.992435088066_dp, & ! Bif. Hopf I
                                                     -72.639047957399_dp, & ! Bif. SN I
                                                     -68.659323383446_dp, & ! Bif. SN II
                                                     -45.225234297115_dp/)  ! Bif. Hopf II

!                                                                                                                    :::::::::::::::::::::::::::::
! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
! ::::::::::::::::::::::::::::: VARIABLES AUXILIARES

! Estas variables son usadas como muletillas computacionales, siendo irrelevante su valor.

real(kind=dp), dimension(3) :: null_farray3
real(kind=dp), dimension(2) :: null_farray2
real(kind=dp)               :: null_float

integer, dimension(3)       :: null_iarray3
integer                     :: null_int

!                                                                                                                    :::::::::::::::::::::::::::::
! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

end module m_parametros
