program analisis_dinamico

! ::: PROGRAMA: ANÁLISIS DINÁMICO ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
! 
!      Este fichero contiene el programa para el análisis dinámico del modelo de Hodgkin-Huxley. El programa proporciona herramientas de cálculo
!   para el estudio de los puntos de equilibrio y bifurcaciones presentes en el modelo de Hodgkin-Huxley reducido. Tal y como se estudia en el 
!   trabajo, el parámetro de bifurcación considerado en los cálculos es E_K.
!
! ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

use m_parametros
use m_modelo_HH_2D
use m_analisis_dinamico
use m_subrutinas

! --- Variables generales: ---
real(kind=dp)               :: t, TMAX, h, V_mem, tr_J, det_J, dv_I, a, c, temp1, temp2
real(kind=dp), dimension(2) :: vp_r, vp_i

! --- Clasificación puntos de equilibrio: ---
integer               :: num_ptos_eq, tipo_pto_eq
integer, dimension(3) :: info_ptos_eq

! --- Bif. SN de ciclos límite: ---
real(kind=dp)                              :: n_max_inest, n_min_inest, n_max_est, n_min_est, paso_adaptable
real(kind=dp), dimension(:,:), allocatable :: fold_bif_data

! --- Variables DFT: ---
real(kind=dp), dimension(2)                 :: y, y_rest
real(kind=dp), dimension(:), allocatable    :: array_Iext, array_Vmem, freq, PSD
complex(kind=dp), dimension(:), allocatable :: array_Iext_hat, array_Vmem_hat, array_impedancia_hat
integer, dimension(:), allocatable          :: L
real(kind=dp)                               :: intensidad, dt
integer                                     :: n

! --- Variables auxiliares: ---
integer                                  :: IOSTAT, i, j
character(len=256)                       :: str_variable
real(kind=dp), dimension(3)              :: xpos_array, ypos_array
real(kind=dp), dimension(:), allocatable :: temp_array

print *
print *, "-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_"
print *, "-_                                                                                                                -_"
print *, "-_   ANÁLISIS DINÁMICO                                                                                            -_"
print *, "-_   -----------------                                                                                            -_"
print *, "-_                                                                                                                -_"
print *, "-_   Indicar una acción con un número entero:                                                                     -_"
print *, "-_                                                                                                                -_"
print *, "-_      (1) Cálculo de puntos de equilibrio + estabilidad                                                         -_"
print *, "-_      (2) Bifurcación de Hopf                                                                                   -_"
print *, "-_            - Cálculo de los valores de bifurcación de E_K                                                      -_"
print *, "-_            - Análisis numérico y gráficas                                                                      -_"
print *, "-_      (3) Bifurcación SN                                                                                        -_"
print *, "-_            - Cálculo de los valores de bifurcación de E_K                                                      -_"
print *, "-_            - Análisis numérico y gráficas                                                                      -_"
print *, "-_      (4) Bifurcación SN de ciclos límite                                                                       -_"
print *, "-_            - Cálculo del valor de bifurcación de E_K                                                           -_"
print *, "-_            - Gráfica                                                                                           -_"
print *, "-_      (5) Representación del espacio de fases                                                                   -_"
print *, "-_      (6) Cálculo de la frecuencia natural de la neurona                                                        -_"
print *, "-_                                                                                                                -_"
print *, "-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_"
print *
read  *, IOSTAT

IOSTAT = IOSTAT + 9

select case(IOSTAT)

case(10)

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "     CÁLCULO DE PUNTOS DE EQUILIBRIO + ESTABILIDAD"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *
     print *, "      Los puntos de equilibrio vienen representados por los puntos de corte de las isoclinas      "
     print *, "   cero en el espacio de fases. En concreto, la isoclina cero V_mem es dependiente del parámetro  "
     print *, "   de bifurcación E_K por lo que primero es necesario indicar el valor de E_K deseado para el     "
     print *, "   estudio del punto de equilibrio.                                                               "
     print *
     print *, "      Introducir el valor del parámetro de bifurcación E_K (en mV):                               "
     print *
     read  *, E_K

     ! --- Cálculos para las gráficas de las isoclinas cero: ---
     open(unit=1, file="isoclinas_cero_temp.txt", status="replace", action="write")
     V_mem = -80.0_dp
     do
          if (V_mem > 60.0_dp) then
               exit
          end if
          write(unit=1,fmt=*) V_mem, HH_isoclina_n_2D(V_mem)
          V_mem = V_mem + 0.1_dp
     end do
     write(unit=1,fmt=*)
     write(unit=1,fmt=*)

     V_mem = -80.0_dp
     i = 1 ! True
     do
          if (V_mem > 60.0_dp) then
               write(unit=1,fmt=*) temp1 + 0.1_dp, 0.0_dp
               exit
          end if

          if (HH_isoclina_V_2D(V_mem) > PRECISION_SP) then
               if (i == 1) then
                    write(unit=1,fmt=*) V_mem, 1.0_dp
                    i = 0 !False
               end if
               write(unit=1,fmt=*) V_mem, HH_isoclina_V_2D(V_mem)
               temp1 = V_mem
          end if

          V_mem = V_mem + 0.1_dp
     end do
     close(unit=1)

     ! --- Cálculo de 1ra aprox. para el punto de equilibrio: ---
     call gnu_file(IOSTAT)

     print *
     print *, "      A continuación se representará el espacio de fases con las isoclinas cero correspondientes  "
     print *, "   al valor de E_K dado. Indicar con el ratón en la gráfica el punto de equilibrio deseado para   "
     print *, "   obtener una 1ª aproximación del mismo. Tras la introducción del punto el programa procederá    "
     print *, "   con los cálculos necesarios para su determinación exacta y para el estudio de la estabilidad.  "
     print *
     print *, "      :: Pulsar ENTER para continuar ::                                                           "
     read  *

     call system("gnuplot gnu_temp.gnu")

     open(unit=1, file="var_temp.txt", status="old", action="read", position="rewind")
     do
          read(unit=1, fmt="(a)") str_variable
          if (str_variable(1:2) == "m_") then
               call string_to_number(str_variable,xpos_array)
               read(unit=1, fmt="(a)") str_variable
               call string_to_number(str_variable,ypos_array)
               exit
          end if
     end do
     close(unit=1)

     call system("rm isoclinas_cero_temp.txt")
     call system("rm gnu_temp.gnu")
     call system("rm var_temp.txt")

     ! --- Cálculo de resultados finales: ---
     print *
     print *
     print *, "   Calculando..."

     pto_eq = (/xpos_array(1), ypos_array(1)/)
     call pto_equilibrio(pto_eq,tr_J,det_J,vp_r,vp_i,PRECISION_SP)

     print *
     print *
     print *
     print *,                         "   --- Punto de equilibrio: ---"
     print *,                         "   --------------------------------------------"
     print "(a, f11.5)",              "      V* = ", pto_eq(1)
     print "(a, f11.5)",              "      n* = ", pto_eq(2)
     print *
     print *,                         "   --- Parámetros de estabilidad: ---"
     print *,                         "   --------------------------------------------"
     print "(a, f11.5)",              "      Traza de J(V*,n*)        = ", tr_J
     print "(a, f11.5)",              "      Determinante de J(V*,n*) = ", det_J
     print *
     print "(a, f11.5, a, f11.5, a)", "      λ1 = ", vp_r(1), " +", vp_i(1), "i"
     print "(a, f11.5, a, f11.5, a)", "      λ2 = ", vp_r(2), " +", vp_i(2), "i"
     print *

     call info_pto_equilibrio(tr_J,det_J,tipo_pto_eq)
     if (tipo_pto_eq == 1) then
          print *, "   Según los parámetros resultantes, el punto de equilibrio se clasifica como: Nodo estable"
          print *, "                                                                               ------------"
     else if (tipo_pto_eq == 2) then
          print *, "   Según los parámetros resultantes, el punto de equilibrio se clasifica como: Nodo inestable"
          print *, "                                                                               --------------"
     else if (tipo_pto_eq == 3) then
          print *, "   Según los parámetros resultantes, el punto de equilibrio se clasifica como: Foco estable"
          print *, "                                                                               ------------"
     else if (tipo_pto_eq == 4) then
          print *, "   Según los parámetros resultantes, el punto de equilibrio se clasifica como: Foco inestable"
          print *, "                                                                               --------------"
     else if (tipo_pto_eq == 5) then
          print *, "   Según los parámetros resultantes, el punto de equilibrio se clasifica como: Punto de silla"
          print *, "                                                                               --------------"
     else if (tipo_pto_eq == 6) then
          print *, "   Para el valor dado de E_K, el punto de equilibrio está sufriendo una bifurcación."
          print *, "                                                                        -----------"
     else
          print *, "   Caso degenerado."
     end if

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *

case(11)

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "     BIFURCACIÓN DE HOPF"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *
     print *, "      La bifurcación de Hopf se caracteriza por la interacción entre un ciclo límite y un punto     "
     print *, "   de equilibrio. Para poder determinar el posible valor de E_K en el que un punto de equilibrio    "
     print *, "   sufre esta bifurcación basta con buscar la condición de no hiperbolicidad tr_J(V*,n*,E_K*) = 0.  "
     print *, "   Finalmente, se verifica que se trata de una bifurcación de Hopf estudiando las condiciones de    "
     print *, "   no degeneración y transversalidad.                                                               "
     print *
     print *, "      En el sistema de HH, para el parámetro E_K, surgen dos bifurcaciones de Hopf. En las          "
     print *, "   siguientes gráficas se representa la traza del jacobiano frente al parámetro E_K. Indicar con    "
     print *, "   el ratón en la gráfica correspondiente a la bifurcación deseada el punto donde la traza se       "
     print *, "   anula, dando así una 1ª aproximación del valor de bifurcación de E_K. Con este valor aproximado  "
     print *, "   el programa calculará el valor exacto de bifurcación E_K y el pertinente análisis numérico.      "
     print *
     print *, "      :: Pulsar ENTER para continuar ::                                                             "
     read  *

     allocate(temp_array(6))

     ! --- Cálculo de 1ra aprox. para E_K: ---
     open(unit=1, file="trJ_temp.txt", status="replace", action="write")
     E_K = -77.0_dp
     do
          if (E_K > -40.0_dp) then
               exit
          end if
          write(unit=1,fmt=*) E_K, traza_J(E_K)
          E_K = E_K + 0.01_dp
     end do
     close(unit=1)

     IOSTAT = 15
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     open(unit=1, file="var_temp.txt", status="old", action="read", position="rewind")
     do
          read(unit=1, fmt="(a)") str_variable
          if (str_variable(1:2) == "m_") then
               call string_to_number(str_variable,xpos_array)
               if (xpos_array(1) < -49.0_dp) then
                    xpos_array(1) = xpos_array(1) - 21.3541_dp
               end if
               E_K = xpos_array(1) ! 1ra aprox. para E_K
               exit
          end if
     end do
     close(unit=1)

     call system("rm trJ_temp.txt")
     call system("rm gnu_temp.gnu")
     call system("rm var_temp.txt")

     print *
     print *
     print *, "   Calculando..."

     ! --- Cálculo del valor de bifurcación para E_K: ---
     if (E_K < -60.0_dp) then
          pto_eq = (/-62.62232_dp, 0.34491_dp/)
     else
          pto_eq = (/-28.41711_dp, 0.83738_dp/)
     end if

     call biseccion(traza_J,E_K - 5.0_dp,E_K + 5.0_dp,temp_array(1),null_int)
     E_K = temp_array(1)
     marker_coordinates = (/E_K, 0.0_dp/)

     ! --- Cálculos para el análisis numérico: ---
     call bifurcacion_Hopf(pto_eq,E_K,tr_J,det_J,a)

     ! --- Cálculo de los ficheros para las gráficas: ---
     temp_array(1) = E_K
     temp_array(2) = pto_eq(1)
     temp_array(3) = pto_eq(2)
     temp_array(4) = tr_J
     temp_array(5) = det_J

     open(unit=1, file="bifurcacion_hopf.txt", status="replace", action="write")
     if (E_K < -60.0_dp) then
          E_K = -77.0_dp
          temp_array(6) = E_K + 6.0_dp
     else
          E_K = -48.0_dp
          temp_array(6) = E_K + 6.0_dp
     end if
     do
          if (E_K > temp_array(6)) then
               exit
          end if
          write(unit=1,fmt=*) E_K, traza_J(E_K)
          E_K = E_K + 0.01_dp
     end do
     write(unit=1,fmt=*)
     write(unit=1,fmt=*)

     E_K = temp_array(1) - 0.5_dp
     pto_eq = (/temp_array(2), temp_array(3)/)
     do
          if (E_K >= temp_array(1) + 0.5_dp) then
               exit
          end if
          call pto_equilibrio(pto_eq,tr_J,det_J,vp_r,vp_i)
          write(unit=1,fmt=*) E_K, vp_r(1), vp_i(1), vp_r(2), vp_i(2)
          E_K = E_K + 0.0001_dp
     end do
     close(unit=1)

     E_K       = temp_array(1)
     pto_eq(1) = temp_array(2)
     pto_eq(2) = temp_array(3)
     tr_J      = temp_array(4)
     det_J     = temp_array(5)

     ! --- Creación de las gráficas: ---
     IOSTAT = 16
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     call system("rm gnu_temp.gnu")

     deallocate(temp_array)

     print *
     print *
     print *
     print *,             "   --- Valor de bifurcación: ---"
     print *,             "   --------------------------------------------"
     print "(a, f17.12)", "       Existe una posible bifurcación de Hopf en E_K =", E_K
     print *
     print *,             "   --- Análisis numérico: ---"
     print *,             "   --------------------------------------------"
     print *,             "      No hiperbolicidad:"
     print "(a, f11.5)",  "         tr_J(V*,n*) (= 0)        = ", tr_J
     print "(a, f11.5)",  "         det_J(V*,n*) (= w^2 > 0) = ", det_J
     print *
     print *,             "      No degeneración:"
     print "(a, f11.5)",  "         a (/= 0) = ", a

     if (a > 0.0_dp) then
          str_variable = "Subcrítica (a > 0)"
     else
          str_variable = "Supercrítica (a < 0)"
     end if

     print *,             "        La bifurcación de Hopf es del tipo: ", trim(str_variable)
     print *
     print *,             "      Transversalidad:"
     print *,             "         c'(b) /= 0 (Analizar pendiente de los valores propios)"
     print *
     print *,             "   --- Gráficas: ---"
     print *,             "   --------------------------------------------"
     print *,             "      Output:"
     print *,             "         'bifurcacion_hopf.txt'"
     print *,             "         'tr_J.pdf', 'autovalores.pdf'"
     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *

case(12)

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "     BIFURCACIÓN SN"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *
     print *, "      La bifurcación SN se caracteriza por la colisión entre un punto de silla y un nodo, ocasionando        "
     print *, "   la desaparición de ambos puntos de equilibrio. Para la búsqueda del valor de bifurcación de E_K se define "
     print *, "   en primer lugar un sistema neuronal 1D"
     print *, "           dV_mem/dt = [I_ext - I_inf(V_mem)]/C_mem ≡ I(V_mem,E_K), "
     print *, "   que junto a la condición de equilibrio I(V,E_K) = 0 resulta la expresión E_K(V_mem). Teniendo en cuenta   "
     print *, "   además la condición de no hiperbolicidad "
     print *, "           ∂I(V_mem,b*)/∂V_mem |V_mem* = 0                                                                   "
     print *, "   se deduce que las bifurcaciones SN se dan en los máximos/mínimos de E_K(V_mem)."
     print *
     print *, "   En la siguiente gráfica se representa la ecuación E_K(V_mem). Indicar aproximadamente con el ratón en     "
     print *, "   dicha gráfica el máximo/mínimo de E_K deseado. A partir de este valor aproximado el programa calculará el "
     print *, "   valor exacto de bifurcación E_K y el pertinente análisis numérico."
     print *
     print *, "      :: Pulsar ENTER para continuar ::                                                                      "
     read  *

     ! --- Cálculo del fichero para la gráfica: ---
     open(unit=1, file="bifurcacion_SN.txt", status="replace", action="write")
     V_mem = -80.0_dp
     do
          if (V_mem > 60.0_dp) then
               exit
          end if
          write(unit=1,fmt=*) V_mem, b_V(V_mem), d_b_V(V_mem)
          V_mem = V_mem + 0.01_dp
     end do
     close(unit=1)

     ! --- Cálculo del intervalo V_mem para mét. bisección: ---
     IOSTAT = 17
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     open(unit=1, file="var_temp.txt", status="old", action="read", position="rewind")
     do
          read(unit=1, fmt="(a)") str_variable
          if (str_variable(1:2) == "m_") then
               call string_to_number(str_variable,xpos_array)
               read(unit=1, fmt="(a)") str_variable
               call string_to_number(str_variable,ypos_array)
               exit
          end if
     end do
     close(unit=1)

     call system("rm gnu_temp.gnu")
     call system("rm var_temp.txt")

     ! --- Cálculo del valor de bifurcación de E_K: ---
     print *
     print *
     print *, "   Calculando..."

     temp1 = xpos_array(1) - 5.0_dp
     temp2 = xpos_array(1) + 5.0_dp
     call biseccion(d_b_V,temp1,temp2,xpos_array(1),null_int)

     E_K = b_V(xpos_array(1))
     marker_coordinates = (/xpos_array(1), E_K/)

     ! --- Cálculos numéricos: ---
     if (E_K < -70.0_dp) then
          pto_eq = (/-43.31588_dp, 0.65602_dp/)
     else
          pto_eq = (/-55.29715_dp, 0.46179_dp/)
     end if

     call bifurcacion_SN(pto_eq,E_K,det_J,dv_I,a,c)

     ! --- Creación de la gráfica: ---
     IOSTAT = 18
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     call system("rm gnu_temp.gnu")

     print *
     print *
     print *
     print *,             "   --- Valor de bifurcación: ---"
     print *,             "   --------------------------------------------"
     print "(a, f17.12)", "       Existe una posible bifurcación SN en E_K =", E_K
     print *
     print *,             "   --- Análisis numérico: ---"
     print *,             "   --------------------------------------------"
     print *,             "      No hiperbolicidad:"
     print "(a, f11.5)",  "         det_J(V*,n*) (= 0) = ", det_J
     print "(a, f11.5)",  "         ∂I(V,b*)/∂V |V* (= 0) = ", dv_I
     print *
     print *,             "      No degeneración:"
     print "(a, f11.5)",  "         a (/= 0) = ", a
     print *
     print *,             "      Transversalidad:"
     print "(a, f11.5)",  "         c (/= 0) = ", c
     print *
     print *,             "   --- Gráfica: ---"
     print *,             "   --------------------------------------------"
     print *,             "      Output:"
     print *,             "         'bifurcacion_SN.txt'"
     print *,             "         'bifurcacion_SN.pdf'"
     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *

case(13)

     diagrama_bif_total = .true. ! diagrama_bif_total = .true.  --> Representación para el intervalo E_K = (-77,-40) mV
                                 ! diagrama_bif_total = .false. --> Representación para el intervalo E_K = (-77,-73.5) mV

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "     BIFURCACIÓN SN DE CICLOS LÍMITE"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *
     print *, "      La bifurcación SN de ciclos límite se caracteriza por la colisión entre un ciclo límite          "
     print *, "   estable y uno inestable. El algoritmo de búsqueda del valor de E_K se basa en precisar primero una  "
     print *, "   propiedad característica de los ciclos límite (máx/min de V_mem, amplitud, etc.) y explotarla para  "
     print *, "   hallar el punto donde los ciclos colisionan. En este caso se ha elegido la amplitud de los ciclos   "
     print *, "   límite en la variable n de activación de K+.                                                        "
     print *
     print *, "      :: Pulsar ENTER para continuar ::                                                                "
     read  *
     print *
     print *
     print *, "   Calculando..."

     if (diagrama_bif_total) then
          allocate(fold_bif_data(3,391))
     else
          allocate(fold_bif_data(3,116))
     end if

     ! --- Cálculos numéricos: ---
     E_K = rango(3)
     paso_adaptable = 0.1_dp
     i = 1
     do
          ! --- Cálculo del ciclo límite estable: ---
          TMAX = 1000.0_dp
          h = 0.01_dp
          t = 0.0_dp
          pto_eq = (/-80.0_dp,1.0_dp/)
          do
               if (t > TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
          end do

          TMAX = 100.0_dp
          t = 0.0_dp
          n_min_est = 1.0_dp
          n_max_est = 0.0_dp
          do
               if (t > TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
               if (pto_eq(2) < n_min_est) then
                    n_min_est = pto_eq(2)
               end if
               if (pto_eq(2) > n_max_est) then
                    n_max_est = pto_eq(2)
               end if
          end do

          ! --- Cálculo del ciclo límite inestable: ---
          TMAX = -1000.0_dp
          h = -0.01_dp
          t = 0.0_dp
          pto_eq = (/-63.14880_dp, 0.33702_dp/)
          do
               if (t < TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
          end do
     
          TMAX = -500.0_dp
          t = 0.0_dp
          n_min_inest = 1.0_dp
          n_max_inest = 0.0_dp
          do
               if (t < TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
               if (pto_eq(2) < n_min_inest) then
                    n_min_inest = pto_eq(2)
               end if
               if (pto_eq(2) > n_max_inest) then
                    n_max_inest = pto_eq(2)
               end if
          end do

          ! --- Cálculo de amplitudes y paso adaptativo: ---
          temp1 = n_max_est - n_min_est
          temp2 = n_max_inest - n_min_inest
          if (temp1 - temp2 == 1.0_dp) then ! Se ha sobrepasado el valor de bifurcación; no se guarda el valor de E_K
               E_K = E_K + paso_adaptable
               E_K = E_K / paso_adaptable
               E_K = aint(E_K)
               E_K = E_K * paso_adaptable
               paso_adaptable = paso_adaptable * 0.1_dp
          else ! Todavía no se ha sobrepasado el valor de bifurcación; se guardan los decimales de E_K calculados hasta el momento
               fold_bif_data(1,i) = E_K
               fold_bif_data(2,i) = temp1
               fold_bif_data(3,i) = temp2
               i = i + 1
               E_K = E_K - paso_adaptable
          end if

          if (paso_adaptable < 1.0e-12_dp) then
               exit
          end if
     end do

     if (diagrama_bif_total) then ! Para estos rangos de E_K no hace falta variar el paso
          E_K = rango(3)
          paso_adaptable = 0.1_dp

          do
               ! --- Cálculo del ciclo límite estable: ---
               TMAX = 1000.0_dp
               h = 0.01_dp
               t = 0.0_dp
               pto_eq = (/-80.0_dp,1.0_dp/)
               do
                    if (t > TMAX) then
                         exit
                    end if
                    call rk4(t,pto_eq,HH_modelo_2D,h)
               end do

               TMAX = 100.0_dp
               t = 0.0_dp
               n_min_est = 1.0_dp
               n_max_est = 0.0_dp
               do
                    if (t > TMAX) then
                         exit
                    end if
                    call rk4(t,pto_eq,HH_modelo_2D,h)
                    if (pto_eq(2) < n_min_est) then
                         n_min_est = pto_eq(2)
                    end if
                    if (pto_eq(2) > n_max_est) then
                         n_max_est = pto_eq(2)
                    end if
               end do

               ! --- Cálculo del ciclo límite inestable: ---
               TMAX = -1000.0_dp
               h = -0.01_dp
               t = 0.0_dp
               pto_eq = (/-63.14880_dp, 0.33702_dp/)
               do
                    if (t < TMAX) then
                         exit
                    end if
                    call rk4(t,pto_eq,HH_modelo_2D,h)
               end do
     
               TMAX = -500.0_dp
               t = 0.0_dp
               n_min_inest = 1.0_dp
               n_max_inest = 0.0_dp
               do
                    if (t < TMAX) then
                         exit
                    end if
                    call rk4(t,pto_eq,HH_modelo_2D,h)
                    if (pto_eq(2) < n_min_inest) then
                         n_min_inest = pto_eq(2)
                    end if
                    if (pto_eq(2) > n_max_inest) then
                         n_max_inest = pto_eq(2)
                    end if
               end do

               ! --- Cálculo de amplitudes: ---
               temp1 = n_max_est - n_min_est
               temp2 = n_max_inest - n_min_inest
               if (temp1 - temp2 == 1.0_dp) then ! Se ha sobrepasado el valor de bifurcación; no se guarda el valor de E_K
                    exit
               else ! Todavía no se ha sobrepasado el valor de bifurcación; se guardan los decimales de E_K calculados hasta el momento
                    fold_bif_data(1,i) = E_K
                    fold_bif_data(2,i) = temp1
                    fold_bif_data(3,i) = temp2
                    i = i + 1
                    E_K = E_K + paso_adaptable
               end if
          end do
     end if

     call matrix_quick_sort(fold_bif_data,1,0)
     E_K = fold_bif_data(1,1)
     marker_coordinates = (/E_K, fold_bif_data(2,1)/)

     ! --- Cálculo del fichero para la gráfica: ---
     open(unit=1, file="bifurcacion_SN_ciclos_limite.txt", status="replace", action="write")
     do i = 1, size(fold_bif_data(1,:))
          write(unit=1,fmt=*) fold_bif_data(:,i)
     end do
     close(unit=1)
     deallocate(fold_bif_data)

     ! --- Creación de la gráfica: ---
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     call system("rm gnu_temp.gnu")

     print *
     print *
     print *
     print *,             "   --- Valor de bifurcación: ---"
     print *,             "   --------------------------------------------"
     print "(a, f17.12)", "      Existe una bifurcación SN de ciclos límite en E_K =", E_K
     print *
     print *,             "   --- Gráfica: ---"
     print *,             "   --------------------------------------------"
     print *,             "      Output:"
     print *,             "         'bifurcacion_SN_ciclos_limite.txt'"
     print *,             "         'bifurcacion_SN_ciclos_limite.pdf'"
     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *

case(14)

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "     REPRESENTACIÓN DEL ESPACIO DE FASES"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *
     print *, "      Introducir el valor del parámetro de bifurcación E_K (en mV) para el cual se quiere dibujar el  "
     print *, "   espacio de fases:                                                                                  "
     print *
     read  *, E_K

     open(unit=1, file="espacio_fases.txt", status="replace", action="write")

     ! --- Isoclina cero n: ---
     V_mem = -80.0_dp
     do
          if (V_mem > 60.0_dp) then
               exit
          end if
          write(unit=1,fmt=*) V_mem, HH_isoclina_n_2D(V_mem)

          V_mem = V_mem + 0.1_dp
     end do
     write(unit=1,fmt=*)
     write(unit=1,fmt=*)

     ! --- Isoclina cero V: ---
     V_mem = -80.0_dp
     i = 1 ! True
     do
          if (V_mem > 60.0_dp) then
               write(unit=1,fmt=*) temp1 + 0.1_dp, 0.0_dp
               exit
          end if

          if (HH_isoclina_V_2D(V_mem) > PRECISION_SP) then
               if (i == 1) then
                    write(unit=1,fmt=*) V_mem, 1.0_dp
                    i = 0 !False
               end if
               write(unit=1,fmt=*) V_mem, HH_isoclina_V_2D(V_mem)
               temp1 = V_mem
          end if

          V_mem = V_mem + 0.1_dp
     end do
     write(unit=1,fmt=*)
     write(unit=1,fmt=*)

     close(unit=1)

     ! --- Recogida de puntos de equilibrio: ---
     print *
     print *, "      Para la representación del espacio de fases es necesario dar una 1ª aproximación de todos los   "
     print *, "   puntos de equilibrio (puntos de corte de isoclinas cero). Indicar con el ratón en la siguiente     "
     print *, "   gráfica dichos puntos, teniendo en cuenta las siguientes instrucciones:                            "
     print *
     print *, "      - El programa espera un primer punto nada más iniciarse (mensaje en pantalla 'Esperando         "
     print *, "        punto...'). La introducción correcta del punto se puede verificar gracias al mensaje          "
     print *, "        adicional 'Punto registrado'.                                                                 "
     print *, "      - Después de introducir cada punto el programa espera una de dos posibles entradas de teclado:  "
     print *, "           TABULADOR: Indica al programa que se quiere añadir un punto adicional (se imprime en       "
     print *, "                      pantalla el mensaje 'Esperando punto...').                                      "
     print *, "           ENTER: Indica al programa que se quiere finalizar la recogida de datos; no hay puntos      "
     print *, "                  adicionales.                                                                        "
     print *
     print *, "      Por defecto el programa admite un máximo de 3 puntos de equilibrio; al añadir el 3er punto el   "
     print *, "   programa seguirá automáticamente con su ejecución.                                                 "
     print *
     print *, "      :: Pulsar ENTER para continuar ::"
     read *
     print *
     print *

     IOSTAT = 19 
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     open(unit=2, file="var_temp.txt", status="old", action="read", position="rewind")
     do
          read(unit=2, fmt="(a)") str_variable
          if (str_variable(1:5) == "array") then
               call string_to_number(str_variable,xpos_array)
               read(unit=2, fmt="(a)") str_variable
               call string_to_number(str_variable,ypos_array)
               exit
          end if
     end do
     close(unit=2)

     num_ptos_eq = 0
     do i = 1, size(xpos_array)
          if (xpos_array(i) /= 1000.0_dp) then
               num_ptos_eq = num_ptos_eq + 1
          end if
     end do

     info_ptos_eq = (/0, 0, 0/)
     do i = 1, num_ptos_eq
          pto_eq = (/xpos_array(i), ypos_array(i)/)
          call pto_equilibrio(pto_eq,tr_J,det_J,vp_r,vp_i,PRECISION_SP)
          call info_pto_equilibrio(tr_J,det_J,tipo_pto_eq)

          xpos_array(i) = pto_eq(1)
          ypos_array(i) = pto_eq(2)
          info_ptos_eq(i) = tipo_pto_eq
     end do

     call system("rm gnu_temp.gnu")
     call system("rm var_temp.txt")

     open(unit=1, file="espacio_fases.txt", status="old", action="write", position="append")

     ! --- Ciclo límite estable: ---
     if (E_K >= rango(1) .and. E_K < rango(5)) then
          TMAX = 1000.0_dp
          h = 0.01_dp

          t = 0.0_dp
          pto_eq = (/-80.0_dp,1.0_dp/)

          do
               if (t > TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
          end do

          TMAX = 100.0_dp
          t = 0.0_dp

          write(unit=1,fmt=*) pto_eq(1), pto_eq(2)
          do
               if (t > TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
               write(unit=1,fmt=*) pto_eq(1), pto_eq(2)
          end do
          write(unit=1,fmt=*)
          write(unit=1,fmt=*)
     end if

     ! --- Ciclo límite inestable: ---
     if (E_K >= rango(1) .and. E_K < rango(2)) then
          TMAX = -1000.0_dp
          h = -0.01_dp
          t = 0.0_dp
          pto_eq = (/xpos_array(1), ypos_array(1)/)
          do
               if (t < TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
          end do

          TMAX = -500.0_dp
          t = 0.0_dp

          write(unit=1,fmt=*) pto_eq(1), pto_eq(2)
          do
               if (t < TMAX) then
                    exit
               end if
               call rk4(t,pto_eq,HH_modelo_2D,h)
               write(unit=1,fmt=*) pto_eq(1), pto_eq(2)
          end do
          write(unit=1,fmt=*)
          write(unit=1,fmt=*)
     end if

     close(unit=1)

     ! --- Generación de gráfica: ---
     print *
     print *
     print *
     print *, "   Dibujando espacio de fases..."

     IOSTAT = 20
     call gnu_file(IOSTAT,E_K,num_ptos_eq,xpos_array,ypos_array,info_ptos_eq)
     call system("gnuplot gnu_temp.gnu")

     call system("rm gnu_temp.gnu")

     print *
     print *
     print *
     print *, "   --- Gráficas: ---"
     print *, "   --------------------------------------------"
     print *, "      Output:"
     print *, "         'espacio_fases.txt'"
     print *, "         'espacio_fases.pdf'"
     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *

case(15)

     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "     CÁLCULO DE LA FRECUENCIA NATURAL DE LA NEURONA"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *
     print *, "      Las neuronas pueden clasificarse como resonadoras o integradoras, dependiendo de si presentan     "
     print *, "   oscilaciones subumbrales del potencial de membrana o no, respectivamente. En concreto, las neuronas  "
     print *, "   resonadoras serán más propensas a disparar potenciales de acción cuando la frecuencia del estímulo   "
     print *, "   dado coincida con su frecuencia natural. Para buscar dicha frecuencia se introduce un estímulo de    "
     print *, "   la forma                                                                                             "
     print *, "           I(t) = 0.2·sin(αt^β)                                                                         "
     print *, "   y se busca la frecuencia predominante aplicando a la variación resultante del potencial de membrana  "
     print *, "   la transformada de Fourier discreta.                                                                 "
     print *
     print *, "      Para el cálculo de la frecuencia el programa asume una situación de reposo neuronal en condiciones"
     print *, "   fisiológicas normales, con E_K = -77 mV.                                                             "
     print *
     print *, "      :: Pulsar ENTER para continuar ::                                                                 "
     read *

     ! --- Cálculo de estímulo y respuesta neuronal: ---

     E_K = -77.0_dp
     call HH_cond_iniciales_2D(E_K,y_rest)
     TMAX = 1024.0_dp
     h = 1.0_dp

     t = 0.0_dp
     call HH_cond_iniciales_2D(E_K,y)

     open(unit=1, file="DFT.txt", status="replace", action="write")
     do
          if (t >= TMAX) then
               exit
          end if
          write(unit=1,fmt=*) t, y(1) - y_rest(1)

          intensidad = 0.2_dp * sin( (2.0_dp * acos(-1.0_dp) * 10.0_dp**(-7.0_dp)) * (t**3.0_dp) )
          call HH_estimulo(t=t,I_externa=intensidad)
          call rk4(t,y,HH_modelo_2D,h)
     end do

     write(unit=1,fmt=*)
     write(unit=1,fmt=*)

     t = 0.0_dp
     n = 0
     do
          if (t >= TMAX) then
               exit
          end if
          intensidad = 0.2_dp * sin( (2.0_dp * acos(-1.0_dp) * 10.0_dp**(-7.0_dp)) * (t**3.0_dp) )
          write(unit=1,fmt=*) t, intensidad
          n = n + 1 ! Número de muestras para DFT

          t = t + 1.0_dp
     end do
     close(unit=1)

     ! --- Cálculo de la impedancia de entrada: ---

     dt = 0.001_dp ! En segundos, para calcular la frecuencia en Hz
     allocate(array_Iext(n),array_Vmem(n),freq(n),PSD(n),array_Iext_hat(n),array_Vmem_hat(n), &
              array_impedancia_hat(n),L(floor(n/2.0_dp)))

     ! Se rellenan los vectores a transformar:
     open(unit=1, file="DFT.txt", status="old", action="read", position="rewind")
     do j = 1, n
          read(unit=1,fmt=*) t, V_mem
          array_Vmem(j) = V_mem
     end do

     read(unit=1,fmt=*)
     read(unit=1,fmt=*)

     do j = 1, n
          read(unit=1,fmt=*) t, I_ext
          array_Iext(j) = I_ext
     end do
     close(unit=1)

     ! Cálculo del eje de frecuencias:
     do j = 1, n
          freq(j) = ( 1 / (dt * n) ) * j
     end do

     do j = 1, size(L)
          L(j) = j
     end do

     ! Cálculo de DFT:
     array_Iext_hat = DFT(array_Iext)
     array_Vmem_hat = DFT(array_Vmem)

     ! Cálculo de la impedancia de entrada:
     array_impedancia_hat = array_Vmem_hat / array_Iext_hat
     PSD = ( array_impedancia_hat * conjg(array_impedancia_hat) ) / n

     ! Escritura de DFT:
     open(unit=1, file="DFT.txt", status="old", action="write", position="append")
     write(unit=1,fmt=*)
     write(unit=1,fmt=*)
     do j = 1, size(L)
          write(unit=1,fmt=*) freq(L(j)), PSD(j)
     end do
     close(unit=1)

     ! --- Creación de las gráficas: ---
     IOSTAT = 23
     call gnu_file(IOSTAT)
     call system("gnuplot gnu_temp.gnu")

     call system("rm gnu_temp.gnu")

     print *
     print *
     print "(a, f8.5, a)", "    La frecuencia natural de la neurona es: ", freq(maxloc(PSD)), " Hz"
     print *
     print *, "   --- Gráficas: ---"
     print *, "   --------------------------------------------"
     print *, "      Output:"
     print *, "         'DFT.txt'"
     print *, "         'DFT.pdf'"
     print *
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *, "--------------------------------------------------------------------------------------------------------------------"
     print *

case default

     print *
     print *, "* * * * * * * * * * * * * * * * * * * *"
     print *, "*  ERROR IOSTAT: Acción no válida     *"
     print *, "* * * * * * * * * * * * * * * * * * * *"
     print *

end select

end program analisis_dinamico
